package kr.co.tadadang.user.native_bridge

import android.app.Activity
import android.webkit.WebView
import com.google.firebase.messaging.FirebaseMessaging
import kr.co.tadadang.user.Const
import okhttp3.OkHttpClient
import retrofit2.Call
import retrofit2.Callback
import retrofit2.Retrofit
import retrofit2.converter.gson.GsonConverterFactory
import retrofit2.http.Body
import retrofit2.http.Headers
import retrofit2.http.PUT

class GetPushToken : _BaseBridge {
    override val cmd: String
        get() = "getPushToken"

    override fun startAction(
        activity: Activity?,
        webView: WebView?,
        data: String?,
        callback: String?
    ) {
        /*
                if (callback != null) {
                    FirebaseMessaging.getInstance().token.addOnCompleteListener(OnCompleteListener { task ->
                        if (activity != null && webView != null) {
                            activity.runOnUiThread {
                                if (!task.isSuccessful) {
                                    webView.loadUrl("javascript:${callback}('');")
                                } else {
                                    val token = task.result
                                    webView.loadUrl("javascript:${callback}('${token}');")
                                }
                            }
                        }
                    })
                }
        */

        if (!data.isNullOrEmpty() && !callback.isNullOrEmpty())
            pushToken(data, callback.replace("getPushToken_CALLBACK_", ""))
    }

    // push token
    private fun pushToken(auth: String, callback: String) {
        FirebaseMessaging.getInstance().token.addOnCompleteListener { task ->
            if (task.isSuccessful) {
                val token = task.result
                sendTokenToServer(auth, token, callback)
            }
        }
    }

    private fun sendTokenToServer(auth: String, token: String, callback: String) {
        val client = OkHttpClient.Builder().addInterceptor { chain ->
            val original = chain.request()
            val requestBuilder = original.newBuilder()
                .header("Authorization", "Bearer $auth")
                .header("req-uuid", callback)
                .method(original.method(), original.body())
            val request = requestBuilder.build()
            chain.proceed(request)
        }.build()

        val retrofit = Retrofit.Builder()
            .baseUrl(Const.PAGE_URL + "/api/")
            .client(client)
            .addConverterFactory(GsonConverterFactory.create())
            .build()

        val apiService = retrofit.create(ApiService::class.java)
        val tokenRequest = TokenRequest(token, "android")

        apiService.sendToken(tokenRequest).enqueue(object : Callback<Void> {
            override fun onResponse(call: Call<Void>, response: retrofit2.Response<Void>) {
                if (response.isSuccessful) {
                }
            }

            override fun onFailure(call: Call<Void>, t: Throwable) {
            }
        })
    }

    data class TokenRequest(val token: String, val os: String)

    interface ApiService {
        @Headers("api-path: adm/push")
        @PUT(".")
        fun sendToken(@Body request: TokenRequest): Call<Void>
    }
    //
}